import 'package:example/tasks_list_page.dart';
import 'package:flutter/material.dart';

import 'firebase_provider.dart';
import 'locator.dart';

class LoginPage extends StatefulWidget {
  LoginPage({Key? key}) : super(key: key);

  @override
  _LoginPageState createState() => _LoginPageState();
}

class _LoginPageState extends State<LoginPage> {
  final FirebaseProvider _fires = locator<FirebaseProvider>();

  final TextEditingController emailController = TextEditingController();
  final TextEditingController passwordController = TextEditingController();

  @override
  void initState() {
    super.initState();
  }

  Future<bool> _login() async {
    try {
      final user =
          await _fires.app.login(emailController.text, passwordController.text);
      if (user != null) {
        _fires.patchState(RootStateObject(userId: user.uid));
      }
      return user != null;
    } catch (e) {
      print(e);
      return false;
    }
  }

  Future<bool> _signupUser() async {
    try {
      final user = await _fires.app
          .signup(emailController.text, passwordController.text);
      return user != null;
    } catch (e) {
      print(e);
      return false;
    }
  }

  Future<bool> _checkLoggedIn() async {
    try {
      final isLoggedIn = await _fires.app.isLoggedIn$().first;
      if (isLoggedIn) {
        final user = await _fires.app.currentUser$().first;
        if (user != null) {
          _fires.patchState(RootStateObject(userId: user.uid));
        }
      }
      return isLoggedIn;
    } catch (e) {
      print(e);
      return false;
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          TextField(
            controller: emailController,
            keyboardType: TextInputType.emailAddress,
          ),
          TextField(
            controller: passwordController,
            keyboardType: TextInputType.text,
            obscureText: true,
          ),
          TextButton(
            child: Text('Log In'),
            onPressed: () async {
              final result = await _login();
              if (result) {
                Navigator.of(context).pushReplacement(
                  MaterialPageRoute<void>(
                    builder: (BuildContext context) => TasksListPage(),
                  ),
                );
              }
            },
          ),
          TextButton(
            child: Text('Sign Up'),
            onPressed: () {
              _signupUser();
            },
          ),
          TextButton(
            child: Text('Check logged In'),
            onPressed: () async {
              final isLoggedIn = await _checkLoggedIn();
              if (isLoggedIn) {
                Navigator.of(context).pushReplacement(
                  MaterialPageRoute<void>(
                    builder: (BuildContext context) => TasksListPage(),
                  ),
                );
              }
            },
          ),
        ],
      ),
    );
  }
}
