import 'dart:math';

import 'package:example/firebase_provider.dart';
import 'package:example/locator.dart';
import 'package:example/login_page.dart';
import 'package:flutter/material.dart';

class TasksListPage extends StatefulWidget {
  @override
  _TasksListPageState createState() => _TasksListPageState();
}

class _TasksListPageState extends State<TasksListPage> {
  final FirebaseProvider _fires = locator<FirebaseProvider>();
  final random = Random();

  @override
  void initState() {
    super.initState();
  }

  void _updateTask(String id) async {
    print('Update: $id');
    final randomNumber = random.nextInt(100);
    await _fires.db.fromCollection(CollectionPaths.UserTasks).update(
        id,
        {
          'id': id,
          'name': 'Task $randomNumber',
        },
        true);
  }

  void _deleteTask(String id) async {
    print('Delete: $id');
    await _fires.db.fromCollection(CollectionPaths.UserTasks).delete(id);
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text('Tasks'),
        actions: [
          IconButton(
            onPressed: () async {
              await _fires.app.logout();
              Navigator.of(context).pushReplacement(
                MaterialPageRoute<void>(
                  builder: (BuildContext context) => LoginPage(),
                ),
              );
            },
            icon: Icon(Icons.logout),
          )
        ],
      ),
      floatingActionButton: FloatingActionButton(
        child: Icon(Icons.add),
        onPressed: () async {
          final randomNumber = random.nextInt(100);
          final task = CustomTask(id: '', name: 'Task $randomNumber');
          final ref = await _fires.db
              .fromCollection(CollectionPaths.UserTasks)
              .add(task.toJson());
          print('Created: ${ref.id}');
        },
      ),
      body: Column(
        children: <Widget>[
          StreamBuilder<List<CustomTask>>(
            stream: _fires.db
                .fromCollection(CollectionPaths.UserTasks)
                .getAllDocsSnap<CustomTask>(
                    (json) => CustomTask.fromJson(json)),
            builder: (context, snap) {
              if (snap.hasError) {
                return Center(child: Text('Error'));
              }
              if (!snap.hasData) {
                return Center(child: CircularProgressIndicator());
              }

              final data = snap.data ?? [];

              return ListView.builder(
                shrinkWrap: true,
                itemCount: data.length,
                itemBuilder: (context, index) {
                  final tasks = data;
                  final task = tasks.elementAt(index);
                  return ListTile(
                    title: Text(task.name),
                    leading: IconButton(
                      onPressed: () => _updateTask(task.id),
                      icon: Icon(Icons.refresh),
                    ),
                    trailing: IconButton(
                      onPressed: () => _deleteTask(task.id),
                      icon: Icon(Icons.delete),
                    ),
                  );
                },
              );
            },
          ),
        ],
      ),
    );
  }
}

class CustomTask {
  final String id;
  final String name;
  CustomTask({required this.id, required this.name});

  factory CustomTask.fromJson(Map<String, dynamic> json) {
    return CustomTask(id: json['id'], name: json['name']);
  }

  Map<String, dynamic> toJson() {
    return {'id': this.id, 'name': this.name};
  }
}
