import 'package:cloud_firestore/cloud_firestore.dart';

import '../utils/firestore_helpers.dart';
import '../query_state.dart';

typedef Query QueryFn(CollectionReference ref);

Stream<List<Map<String, dynamic>>> collectionQueryGetAllDocs(QueryState q,
    [QueryFn? whereQuery]) {
  return q.refCollection().take(1).map((collection) {
    if (whereQuery != null) {
      return whereQuery(collection);
    }
    return collection;
  }).asyncMap((collection) async {
    try {
      final res = await collection.get();
      return res.docs;
    } catch (error) {
      return [] as List<QueryDocumentSnapshot>;
    }
  }).asyncMap((docs) => q.docArray2Data(docs));
}

Stream<List<Map<String, dynamic>>> collectionQueryGetAllDocsSnap(QueryState q,
    [QueryFn? whereQuery]) {
  return q
      .refCollection()
      .map((collection) {
        if (whereQuery != null) {
          return whereQuery(collection);
        }
        return collection;
      })
      .asyncExpand((collection) => collectionSnap2Stream(collection))
      .map((docChanges) => docChanges.docs)
      .asyncMap((docs) => q.docArray2Data(docs));
}

Stream<Map<String, dynamic>> collectionQueryGetId(QueryState q, String id) {
  return q
      .refCollection()
      .take(1)
      .map((collection) => collection.doc(id))
      .asyncMap((doc) => doc.get())
      .asyncMap(
        (doc) => q.doc2Data(doc),
      );
}

Stream<Map<String, dynamic>> collectionQueryGetIdSnap(QueryState q, String id) {
  final $collection = q.refCollection();

  final $resultResolved = $collection
      .map((collection) => collection.doc(id))
      .asyncExpand((doc) => documentSnap2Stream(doc))
      .asyncMap((doc) => q.doc2Data(doc));

  return $resultResolved;
}
