import 'package:firebase_auth/firebase_auth.dart';
import 'package:firebase_core/firebase_core.dart';

import 'firebase_wrapper.dart';
import 'firestore_state_manager.dart';
import 'firestore_wrapper.dart';

class FirebaseClient<C, D> {
  final _stateManager = FirestoreStateManager();
  late FirebaseWrapper<C, D> _firebaseWrapper;

  FirebaseApp _firebaseApp;

  FirebaseClient(this._firebaseApp) {
    this._firebaseWrapper = FirebaseWrapper(_firebaseApp, this._stateManager);
  }

  Map<String, dynamic>? get getRootState => this._stateManager.all;

  FirestoreWrapper<C, D> get db => this._firebaseWrapper.provider;

  void patchRootState(Map<String, dynamic> state) {
    this._stateManager.patchState(state);
  }

  Future<User?> signup(String email, String password) async {
    try {
      final userRecord = await this._firebaseWrapper.signup(email, password);
      return userRecord;
    } catch (error) {
      throw Exception(error);
    }
  }

  Future<User?> login(String email, String password) async {
    try {
      final userRecord = await this._firebaseWrapper.login(email, password);
      return userRecord;
    } catch (error) {
      throw Exception(error);
    }
  }

  Future<void> logout() {
    return this._firebaseWrapper.logout();
  }

  Stream<User?> currentUser$() {
    final authState$ = this._firebaseWrapper.getAuthUser$();
    return authState$;
  }

  Stream<bool> isLoggedIn$() {
    return this._stateManager.user$.map((user) => user != null);
  }
}
