import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:firebase_core/firebase_core.dart';
import 'firestore_state_manager.dart';
import 'firestore_wrapper.dart';

class FirebaseWrapper<C, D> {
  final FirebaseApp app;
  final auth = FirebaseAuth.instance;
  final db = FirebaseFirestore.instance;

  final FirestoreStateManager rootState;
  late FirestoreWrapper<C, D> provider;

  FirebaseWrapper(
    this.app,
    this.rootState,
  ) {
    this.provider = FirestoreWrapper(app, rootState);
    this._initFirebaseFromCache();
  }

  Future<void> _initFirebaseFromCache() async {
    try {
      await _initUserFromAuth();
    } catch (error) {
      print(error);
    }
    this.rootState.initializationDone();
  }

  Future<void> _initUserFromAuth() async {
    final authState$ = this.auth.authStateChanges();
    final user = await authState$.first;
    if (user == null) {
      throw Exception('User is not previously logged in');
    }
    this.rootState.patchState({'user': user, 'uid': user.uid});
  }

  Future<User?> signup(String email, String password) async {
    try {
      final authUser = await this
          .auth
          .createUserWithEmailAndPassword(email: email, password: password);
      return authUser.user;
    } on FirebaseAuthException catch (e) {
      if (e.code == 'weak-password') {
        print('The password provided is too weak.');
      } else if (e.code == 'email-already-in-use') {
        print('The account already exists for that email.');
      }
    } catch (e) {
      print(e);
    }
  }

  Future<User?> login(String email, String password) async {
    try {
      final authUser = await this
          .auth
          .signInWithEmailAndPassword(email: email, password: password);
      final user = await auth.authStateChanges().first;
      this.rootState.patchState({'user': user});
      return authUser.user;
    } catch (e) {
      throw Exception('invalid credentials...');
    }
  }

  Future<void> logout() {
    this.rootState.clearState();
    return this.auth.signOut();
  }

  Stream<User?> getAuthUser$() {
    final authState$ = this.auth.authStateChanges();
    return authState$;
  }
}
