import 'package:firebase_auth/firebase_auth.dart';
import 'package:rxdart/rxdart.dart';

class FirestoreStateManager {
  final _root = BehaviorSubject<Map<String, dynamic>>();
  final _hasBeenInitialized = BehaviorSubject<bool>();

  Stream<Map<String, dynamic>> $all() {
    return Rx.combineLatest2(
        this._root.stream,
        this.$hasDefinitelyInitialized(),
        (Map<String, dynamic> root, bool init) => root);
  }

  Map<String, dynamic>? get all => _root.value;

  String get currentUid {
    final currentRoot = this._root.value;
    if (currentRoot == null || currentRoot['user'] == null) {
      throw Exception('Not logged in yet, couldnt get user');
    }
    return currentRoot['user'].uid;
  }

  Stream<User?> get user$ {
    return Rx.combineLatest2(
        this._root,
        this.$hasDefinitelyInitialized(),
        (Map<String, dynamic>? a, b) =>
            a != null && a['user'] != null ? a['user'] as User : null);
  }

  void patchState(Map<String, dynamic> state) {
    final stateMap = state;
    final currentMap = all ?? {};
    for (var e in stateMap.entries) {
      currentMap[e.key] = e.value;
    }
    final newState = currentMap;
    print(newState);
    this._root.add(newState);
  }

  void initializationDone() {
    this._hasBeenInitialized.add(true);
  }

  Stream<bool> $hasDefinitelyInitialized() {
    return this._hasBeenInitialized.where((i) => !!i);
  }

  void dispose() {
    _root.close();
    _hasBeenInitialized.close();
  }

  clearState() {
    this._root.add({});
  }
}
