import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:firebase_core/firebase_core.dart';
import 'package:flutter_firecache/src/query_state.dart';

import 'builders/collection_builder.dart';
import 'builders/document_builder.dart';
import 'firestore_state_manager.dart';

class FirestoreWrapper<C, D> {
  final FirebaseApp _firebaseApp;
  final FirestoreStateManager rootState;
  late FirebaseFirestore app;
  late FirebaseAuth auth;

  FirestoreWrapper(
    this._firebaseApp,
    this.rootState,
  ) {
    this.app = FirebaseFirestore.instanceFor(app: _firebaseApp);
    this.auth = FirebaseAuth.instanceFor(app: _firebaseApp);
    this.rootState.initializationDone();
  }

  CollectionQueryBuilder fromCollection(String collectionPathTemplate) {
    final queryState = QueryState(
      this.rootState,
      (collectionPathTemplate as dynamic) as String,
      this.app,
    );
    return CollectionQueryBuilder(queryState);
  }

  DocumentQueryBuilder fromDocument(String documentPathTemplate) {
    final queryState = QueryState(
      this.rootState,
      documentPathTemplate,
      this.app,
    );
    return DocumentQueryBuilder(queryState);
  }

  String getRandomDocId() {
    return this.app.collection("collections").doc().id;
  }
}
