import 'package:cloud_firestore/cloud_firestore.dart';

import 'firestore_state_manager.dart';
import 'utils/path_resolver.dart';

class QueryState {
  final FirestoreStateManager appState$;
  Map<String, dynamic>? overridenState;
  final String pathTemplate;
  final FirebaseFirestore app;
  QueryState(this.appState$, this.pathTemplate, this.app);

  final bool _enableUpdateFields = true;
  final bool _disableUpdateFields = false;

  String get uid {
    return this.appState$.currentUid;
  }

  Stream<DocumentReference> refDocument() {
    return resolvePathVariables(this).map((documentPath) {
      if (documentPath.isNotEmpty) {
        return this.app.doc(documentPath);
      }
      throw Exception("No document path could not be resolved!!");
    });
  }

  Stream<CollectionReference> refCollection() {
    return resolvePathVariables(this).map((collectionPath) {
      if (collectionPath.isEmpty) throw Exception();
      return this.app.collection(collectionPath);
    });
  }

  Future<Map<String, dynamic>> doc2Data(DocumentSnapshot doc) async {
    final dataSafe = await this._getDocData(doc);
    dataSafe['id'] = doc.id;
    return dataSafe;
  }

  Future<Map<String, dynamic>> _getDocData(DocumentSnapshot doc) async {
    final dataSafe = doc.data() ?? {};
    return dataSafe;
  }

  Future<List<Map<String, dynamic>>> docArray2Data(
      List<DocumentSnapshot> docs) {
    final promises = docs.map((doc) => this.doc2Data(doc));
    return Future.wait(promises);
  }

  void setUpdatedProps(Map<String, dynamic> obj, String uid) {
    final shouldSetFields = _enableUpdateFields && !_disableUpdateFields;
    if (!shouldSetFields) {
      return;
    }
    obj['updated_at'] = DateTime.now();
    obj['updated_by'] = uid;
  }

  void setCreatedProps(Map<String, dynamic> obj, String uid) {
    final shouldSetFields = _enableUpdateFields && !_disableUpdateFields;
    if (!shouldSetFields) {
      return;
    }
    obj["created_at"] = DateTime.now();
    obj["created_by"] = uid;
  }

  QueryState overrideAppState(Map<String, dynamic> overridenState) {
    this.overridenState = overridenState;
    return this;
  }
}
